#!/usr/bin/env python3

"""
Post-processing step for STAR Salmon analysis.

This includes

  1. extracting TPM and read information from salmon quant
  2. running MultiQC
"""
import argparse
import json
from rpy2.robjects.packages import importr
import subprocess
import os

DESCRIPTION = """post_star_salmon.py - Post-run step for STAR Salmon"""

if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('configfile', help='configuration file')
    args = parser.parse_args()
    with open(args.configfile) as infile:
        config = json.load(infile)

    # Determine post run directory. If not specified, make it sub directory of
    # output directory
    output_dir = config['output_dir']
    try:
        postrun_outdir = config['postrun_output_dir']
    except:
        postrun_outdir = os.path.join(output_dir, "Post_Run_Results")

    genome_dir = config['genome_dir']
    org1, org2 = os.path.basename(genome_dir).split('_')
    print('\033[33mExtracting salmon quant files...\033[0m')
    global_search = importr("GlobalSearch")
    global_search.extract_salmon_quants(org1, org2, output_dir, postrun_outdir)
    # now run MultiQC
    print('\033[33mRunning MultiQC...\033[0m')
    multiqc_outdir = os.path.join(postrun_outdir, 'MultiQC')
    if not os.path.exists(multiqc_outdir):
        os.makedirs(multiqc_outdir)
    command = ['multiqc', '--outdir', multiqc_outdir, output_dir]
    compl_proc = subprocess.run(command, check=True, capture_output=False, cwd=output_dir)



#!/usr/bin/env python3
"""
find_files.py - module for flexible finding of FASTQ files
"""
from collections import defaultdict
import glob
import logging
import os, re, fs
import jinja2
from fs.osfs import OSFS  # make sure we install the fs package !!!


def rnaseq_data_folder_list(config, filesys=OSFS('/')):
    """Function to determine the list of directories that are to be submitted to
    the cluster for RNA sequencing analysis
    """
    result = []
    pattern = re.compile('R[E]?\d+.*')
    try:
        includes_list = config['includes']
    except KeyError:
        includes_list = []
    try:
        include_path = config['include_file']
        if len(include_path) > 0 and filesys.exists(include_path):
            with filesys.open(include_path) as infile:
                for line in infile:
                    includes_list.append(line.strip())
        elif len(include_path) > 0 and not filesys.exists(include_path):
            logging.warning("include file '%s' does not exist", include_path)
    except:
        # ignore any errors in the include_file
        pass

    if len(includes_list) > 0:
        # Take the list specified in the "includes" section of the configuration file
        result = includes_list
    else:
        # take the top level directories in the input directory
        # that match the pattern
        #print("ADD TOPLEVEL FILES: %s" % config['input_dir'])
        #result = [d for d in filesys.listdir(config['input_dir']) if re.match(pattern, d)]
        result = [d for d in filesys.listdir(config['input_dir'])
                if filesys.isdir(fs.path.combine(config['input_dir'], d))]
    return result


def _find_fastq_files(data_folder, patterns, rootfs):
    """Stable version of file finder, search multiple patterns"""
    logger = logging.getLogger("rnaseq")
    result = []
    filesys = rootfs.opendir(data_folder)
    for pat in patterns:
        templ = jinja2.Template(pat)
        patmatch = defaultdict(list)
        for read_num in [1, 2]:
            # keep "pairnum" for legacy reasons
            pattern = templ.render({'pairnum': read_num, 'readnum': read_num})
            pattern = fs.path.combine('/**', pattern)
            #print("SEARCHING PATTERN: '%s'" % pattern, flush=True)
            for match in filesys.glob(pattern):
                patmatch[read_num].append(fs.path.combine(data_folder, match.path))
        #print("PATMATCH: ", patmatch)
        try:
            first = patmatch[1][0]
        except:
            # no match -> skip this pattern
            continue
        try:
            second = patmatch[2][0]
        except:
            second = None
        result.append((first, second))
    return result


def find_fastq_files(data_folder, patterns, filesys=OSFS('/')):
    """
    This function finds the FASTQ files according to the specified patterns. It will
    start at data_folder and try to find all FASTQ files, if possible as pairs.
    If there is no second file, the second component of the result will be None.
    For the most part, patterns will follow the glob format, you can specify the position
    of the pair number in Jinja2 format
    Example:

    A pattern of

    *_{{readnum}}.fastq.gz

    with a readnum of 1 would "match myfile_1.fastq.gz"


    :param data_folder: the top level folder to start searching from
    :param patterns: list of patterns to use in glob searching
    :param filesys: the PyFS file system to use in glob searching
    :return the list of matching paths
    """
    return _find_fastq_files(data_folder, patterns, filesys)


# Base pattern for searching
#DATA_SEARCH1 = '%s/RNA/*R1*.fastq*' % data_folder # test
#DATA_SEARCH1 = '%s/*_1.fq*' % data_folder
DATA_SEARCH1 = "/RNA/*[_.R]1*.fastq.gz"
DATA_SEARCH2 = "/RNA/*/*[_.R]1*.fastq.gz"

def find_fastq_files_experimental(data_folder, pattern1=DATA_SEARCH1, pattern2=DATA_SEARCH2):
    # get logger object
    logger = logging.getLogger("rnaseq")

    # Get the list of first file names in paired end sequences
    print("SEARCHING FIRST PAIRS IN: ", pattern)
    # Search first set of folders
    first_pair_files1 = glob.glob(data_folder + pattern1, recursive=True)
    # search additional folders if there are different levels
    first_pair_files2 = glob.glob(data_folder + pattern2, recursive=True)

    # combine search results
    first_pair_files = first_pair_files1 + first_pair_files2

    if len(first_pair_files) == 0:
        logger.error("Error: I didnt find any file with '*fastq*' extension in %s\n" %(folder_name))

    """
    # Program specific results directories
    data_trimmed_dir = "%s/trimmed" % (results_folder)
    fastqc_dir = "%s/fastqc_results" % (results_folder)

    results_dir = "%s/results_STAR" %(results_folder)
    rsem_results_dir = "%s/results_RSEM" %(results_folder) 

    # Run create directories function to create directory structure
    create_dirs(data_trimmed_dir, fastqc_dir, results_dir, rsem_results_dir)

    print("FIRST_PAIR_FILES: ", first_pair_files)

    # Loop through each file and create filenames
    file_count = 1
    for first_pair_file in first_pair_files:
        first_file_name_full = first_pair_file.split('/')[-1]

        if re.search("_R1_001.fastq.gz", first_pair_file):
            print("Found a match for: '_R1_001.fastq.gz'\n")
            second_pair_file = re.sub("_R1_001.fastq.gz", "_R2_001.fastq.gz", first_pair_file)
        elif re.search(".R1.fastq.gz", first_pair_file):
            print("Found a match for: '.R1.fastq.gz'\n")
            second_pair_file = re.sub(".R1.fastq.gz", ".R2.fastq.gz", first_pair_file)
        elif re.search("_1.fastq.gz", first_pair_file):
            print("Found a match for: '_1.fastq.gz'\n")
            second_pair_file = re.sub("_1.fastq.gz", "_2.fastq.gz", first_pair_file)
        elif re.search(".1.fastq.gz", first_pair_file):
            print("Found a match for: '.1.fastq.gz'\n")
            second_pair_file = re.sub(".1.fastq.gz", ".2.fastq.gz", first_pair_file)
        else:
            print("No match found for any defined types\n")
            with open(error_file,'w') as f:
                f.write("No match found for any defined types\n")
                f.write(str(first_pair_file))

        print(first_pair_file)
        print(second_pair_file)

        second_file_name_full = second_pair_file.split('/')[-1]
        file_ext = first_pair_file.split('.')[-1]
        print()
        print ('\033[32m Processing File: %s of %s (%s)\033[0m' %(file_count, len(first_pair_files), first_file_name_full ))

        first_file_name = re.split('.fq|.fq.gz',first_file_name_full)[0]
        second_file_name = re.split('.fq|.fq.gz',second_file_name_full)[0]
        print('first_file_name:%s, second_file_name:%s' %(first_file_name,second_file_name))
"""

DATA_FOLDER = "/proj/omics4tb2/wwu/GlobalSearch.old/Pilot_Fail_Concat/rawdata"
SEARCH_PATTERN = '/*_1.fq*'
if __name__ == '__main__':
    files = find_fastq_files(os.path.join(DATA_FOLDER, 'R1'), SEARCH_PATTERN)
    print(files)
 



#!/usr/bin/env python3

#############################################################
##### RNASeq Analysis Pipeline with STAR                #####
##### Last update: 11/15/2022 Serdar Turkarslan         #####
##### Institute for Systems Biology                     #####
############################################################
import glob, sys, os, string, datetime, re
import argparse
import subprocess

from .find_files import find_fastq_files
from .trim_galore import trim_galore, collect_trimmed_data, create_result_dirs

DESCRIPTION = """run_STAR_SALMON.py - run STAR and Salmon"""

####################### Run STAR #####################################
### We need to add Read GRoup info
### --outSAMattrRGline ID:${i%_TF_R1_val_1.fq.gz}
### https://github.com/BarshisLab/danslabnotebook/blob/main/CBASSAS_GenotypeScreening.md

def run_star(first_pair_group, second_pair_group, results_dir, folder_name, genome_dir, args):
    print('\033[33mRunning STAR! \033[0m', flush=True)
    outfile_prefix = '%s/%s_%s_' % (results_dir, folder_name, args.starPrefix)
    star_options = ["--runThreadN", str(args.runThreadN),
                    "--outFilterType", "Normal",
                    "--outSAMstrandField", "intronMotif",
                    "--outFilterIntronMotifs", "RemoveNoncanonical",
                    "--outSAMtype", "BAM", "Unsorted",
                    "--limitBAMsortRAM", str(args.limitBAMsortRAM),
                    "--readFilesCommand", "zcat",
                    "--outReadsUnmapped", "Fastx",
                    "--outFilterMismatchNmax", str(args.outFilterMismatchNmax),
                    "--outFilterMismatchNoverLmax", str(args.outFilterMismatchNoverLmax),
                    "--outFilterScoreMinOverLread", str(args.outFilterScoreMinOverLread),
                    "--outFilterMatchNmin", str(args.outFilterMatchNmin)]

    genome_load = "LoadAndKeep"  # This is the default, for efficiency
    if args.twopassMode:
        star_options.extend(["--twopassMode", "Basic"])
        genome_load = "NoSharedMemory"  # two-pass has to run without shared memory

    command = ["STAR", "--genomeDir", genome_dir]
    command += star_options
    if args.outSAMattributes != "Standard" and len(args.outSAMattributes) > 0:
        out_sam_attrs = args.outSAMattributes.split()
        command.append('--outSAMattributes')
        command += out_sam_attrs

    # Handling for GFF files
    if not args.genome_gff is None and os.path.exists(args.genome_gff):
        genome_load = "NoSharedMemory"  # can't use GFF with a shared genome memory
        gff_args = [
            '--sjdbGTFfile', args.genome_gff,
            '--sjdbGTFtagExonParentTranscript', args.sjdbGTFtagExonParentTranscript,
            '--limitSjdbInsertNsj', str(args.limitSjdbInsertNsj)
        ]
        if args.sjdbOverhang is not None:
            gff_args += ['--sjdbOverhang', str(args.sjdbOverhang)]
        command += gff_args

    command += [ "--readFilesIn", first_pair_group,
                 second_pair_group,
                 "--outFileNamePrefix", outfile_prefix]
    command += ["--genomeLoad", genome_load]

    cmd = ' '.join(command)
    compl_proc = subprocess.run(command, check=True, capture_output=False, cwd=results_dir)

####################### Deduplication (not in _old) ###############################
def dedup(results_dir,folder_name):
    print('\033[33mRunning Deduplication! \033[0m', flush=True)
    outfile_prefix = '%s/%s_%s_' %(results_dir, folder_name, args.starPrefix)

    aligned_bam = '%sAligned.out.bam' % (outfile_prefix)
    fixmate_bam = '%sFixmate.out.bam' % (outfile_prefix)
    ordered_bam = '%sOrdered.out.bam' % (outfile_prefix)
    markdup_bam = '%sMarkedDup.out.bam' % (outfile_prefix)
    markdupSTAR_bam = '%sProcessed.out.bam' % (outfile_prefix)
    nosingleton_bam = '%sNoSingleton.out.bam' % (outfile_prefix)
    nosingletonCollated_bam = '%sNoSingletonCollated.out.bam' % (outfile_prefix)

    # STAR mark duplicates
    star_markdup_command = ['STAR', '--runThreadN', '32',
                            '--runMode',
                            'inputAlignmentsFromBAM',
                            '--bamRemoveDuplicatesType', 'UniqueIdenticalNotMulti',
                            '--inputBAMfile', aligned_bam,
                            '--outFileNamePrefix', outfile_prefix]
    star_markdup_cmd = ' '.join(star_markdup_command)

    # removesingletons from STAR
    rmsingletonsSTAR_command = ['samtools', 'view', '-@', '8',
                                '-b', '-F', '0x400', markdupSTAR_bam,
                                '>', nosingleton_bam]
    rmsingletonsSTAR_cmd = ' '.join(rmsingletonsSTAR_command)

    # Collate reads by name
    collatereadsSTAR_command = ['samtools', 'sort', '-o',
                                nosingletonCollated_bam,
                                '-n', '-@', '8', nosingleton_bam]
    collatereadsSTAR_cmd = ' '.join(collatereadsSTAR_command)

    ## STAR based BAM duplicate removal
    # Mark duplicates with STAR
    print('STAR mark duplicates run command:%s' % star_markdup_cmd, flush=True)
    compl_proc = subprocess.run(star_markdup_command, check=True, capture_output=False, cwd=results_dir)

    # Remove marked duplicates withh samtools
    print('Samtools  STAR Dedup Remove run command:%s' % rmsingletonsSTAR_cmd, flush=True)
    compl_proc = subprocess.run(rmsingletonSTAR_command, check=True, capture_output=False, cwd=results_dir)

    # Remove marked duplicates withh samtools
    print('Samtools  Collate reads by read name run command:%s' % collatereadsSTAR_cmd, flush=True)
    compl_proc = subprocess.run(collatereadsSTAR_command, check=True, capture_output=False, cwd=results_dir)


####################### Run Salmon Count ###############################
# WW: Check the names of the input files they will be different from _out
def run_salmon_quant(results_dir, folder_name, genome_fasta):
    outfile_prefix = '%s/%s_%s_' %(results_dir, folder_name, args.starPrefix)
    print(outfile_prefix, flush=True)
    print('\033[33mRunning salmon-quant! \033[0m', flush=True)
    # check if we are performing deduplication
    if args.dedup:
        salmon_input = '%sNoSingletonCollated.out.bam' % (outfile_prefix)
    else:
        salmon_input = '%sAligned.out.bam' % (outfile_prefix)

    command = ['salmon', 'quant', '-t', genome_fasta,
        '-l', 'A',  '-a',  salmon_input, '-o', '%s/%s_salmon_quant' % (results_dir, args.salmonPrefix)]
    cmd = ' '.join(command)
    print("Salmon quant command: '%s'" % cmd, flush=True)
    # run as a joined string
    compl_proc = subprocess.run(cmd, check=True, capture_output=False, cwd=results_dir, shell=True)


####################### Run HTSEq Count ###############################
#### We can remove this since we are using salmon quant
def run_htseq(htseq_dir, results_dir, folder_name, genome_gff):
    print('\033[33mRunning htseq-count! \033[0m', flush=True)
    htseq_input = '%s/%s_star_Aligned.sortedByCoord.out.bam' %(results_dir, folder_name)
    cmd = 'htseq-count -s "reverse" -t "exon" -i "Parent" -r pos --max-reads-in-buffer 60000000 -f bam %s %s > %s/%s_htseqcounts.txt' %(htseq_input,
                                                                                                                                        genome_gff,htseq_dir,folder_name)
    print('htseq-count run command: %s' % cmd, flush=True)
    os.system(cmd)


####################### Running the Pipeline ###############################

def run_pipeline(data_folder, results_folder, genome_dir, genome_fasta, args):
    folder_count = 1

    # Loop through each data folder
    folder_name = data_folder.split('/')[-1]
    print('\033[33mProcessing Folder: %s\033[0m' % folder_name, flush=True)

    # Get the list of first file names in paired end sequences
    ## We need to make sure we capture fastq data files
    pair_files = find_fastq_files(data_folder, args.fastq_patterns.split(','))

    # Program specific results directories
    data_trimmed_dir = "%s/%s/trimmed" % (results_folder,folder_name)
    fastqc_dir = "%s/%s/fastqc_results" % (results_folder,folder_name)

    results_dir = "%s/%s/results_STAR_Salmon" %(results_folder, folder_name)
    htseq_dir = "%s/htseqcounts" % (results_dir)

    # Run create directories function to create directory structure
    create_result_dirs(data_trimmed_dir, fastqc_dir, results_dir, htseq_dir)

    print("PAIR_FILES: ", pair_files, flush=True)

    # Loop through each file and create filenames
    file_count = 1

    is_gzip = True
    is_paired_end = True

    for pair_file in pair_files:
        first_pair_file, second_pair_file = pair_file
        if second_pair_file is None:
            is_paired_end = False

        fastq_fname = os.path.basename(first_pair_file)
        is_gzip = fastq_fname.endswith("gz")
        if is_gzip:
            file_ext = '.'.join(fastq_fname.split('.')[-2:])
        else:
            file_ext = fastq_fname.split('.')[-1]

        print('\033[32m Processing File: %s of %s ("%s")\033[0m' % (file_count, len(pair_files), first_pair_file),
              flush=True)

        # Collect Sample attributes
        sample_id = fastq_fname.replace(file_ext, "")
        print("sample_id: %s" % sample_id, flush=True)

        # Run TrimGalore
        trim_galore(first_pair_file, second_pair_file, folder_name,sample_id, data_trimmed_dir, fastqc_dir)
        file_count += 1

    # Collect Trimmed data for input into STAR
    first_pair_group, second_pair_group = collect_trimmed_data(data_trimmed_dir, is_gzip, is_paired_end)

    # Run STAR
    run_star(first_pair_group, second_pair_group, results_dir, folder_name, genome_dir, args)

    # Run Deduplication
    if args.dedup:
        print('\033[33mRunning Deduplication: \033[0m', flush=True)
        dedup(results_dir,folder_name)

    # Run Salmon Quant
    run_salmon_quant(results_dir, folder_name, genome_fasta)
    folder_count += 1

    return data_trimmed_dir, fastqc_dir, results_dir


if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('genomedir', help='genome directory')
    parser.add_argument('dataroot', help="parent of input directory")
    parser.add_argument('indir', help="input directory (R<somenumber>)")
    parser.add_argument('outdir', help='output directory')
    parser.add_argument('--fastq_patterns', help="FASTQ file patterns", default="*_{{pairnum}}.fq.*")
    parser.add_argument('--genome_gff', help='genome GFF file')
    parser.add_argument('--genome_fasta', help='genome FASTA file')
    parser.add_argument('--dedup', action='store_true', help='should we deduplicate bam files (True or False)')
    parser.add_argument('--twopassMode', action='store_true', help='run STAR in two-pass mode')
    parser.add_argument('--starPrefix', help="STAR output file name prefix")
    parser.add_argument('--salmonPrefix', help="Salmon output folder name prefix")
    parser.add_argument('--outFilterMismatchNmax', nargs='?', const=10, type=int)
    parser.add_argument('--outFilterMismatchNoverLmax', nargs='?', const=0.3, type=float)
    parser.add_argument('--outFilterScoreMinOverLread', nargs='?', const=0.66, type=float)
    parser.add_argument('--outFilterMatchNmin', nargs='?', const=0, type=int)
    parser.add_argument('--outSAMattributes', nargs='?', type=str, default="Standard")
    parser.add_argument('--runThreadN', type=int, default=32)
    parser.add_argument('--limitBAMsortRAM', type=int, default=5784458574)
    parser.add_argument('--sjdbGTFtagExonParentTranscript', default="Parent")
    parser.add_argument('--sjdbOverhang', type=int, default=None)
    parser.add_argument('--limitSjdbInsertNsj', type=int, default=1602710)
    args = parser.parse_args()

    now = datetime.datetime.now()
    timeprint = now.strftime("%Y-%m-%d %H:%M")
    data_folder = "%s/%s" % (args.dataroot, args.indir)
    if args.genome_fasta is not None and os.path.exists(args.genome_fasta):
        genome_fasta = args.genome_fasta
    else:
        genome_fasta = glob.glob('%s/*.fasta' % (args.genomedir))[0]

    data_trimmed_dir,fastqc_dir,results_dir = run_pipeline(data_folder, args.outdir, args.genomedir, genome_fasta, args)



#!/usr/bin/env python3

"""
RNASeq Analysis pipeline using Kallisto
"""
import glob, sys, os, re, subprocess
from .trim_galore import trim_galore, collect_trimmed_data, create_result_dirs
import argparse

# data and results directories
# These were the original examples
#RUN_DIR = "/proj/omics4tb2/Global_Search"
#DATA_DIR = "%s/Pilot_Pass/X204SC21081158-Z01-F003/raw_data" % RUN_DIR
#GENOME_DIR = "%s/reference_genomes/acerv_smic-reefGenomics" % RUN_DIR
#TRANSCRIPTOME_FILE = "%s/Acerv_Smic-ReefGenomics_merged.fasta" % GENOME_DIR

############# Functions ##############
####################### Run Kalisto ###############################
def run_kallisto(index_path, results_dir, pair_files):
    print('\033[33mRunning kallisto! \033[0m')
    # flatten the pair_files list into an input file list
    input_files = []
    for pair_file in pair_files:
        input_files.extend(pair_file)

    command = ['kallisto',
              'quant', '-i', index_path]
    command.extend(input_files)
    command.extend(['-o', results_dir,
                    '-b', '100', '--bias', '-t', '4', '--rf-stranded'])
    kallisto_cmd = ' '.join(command)
    print('Kallisto run command: "%s"' % kallisto_cmd)
    compl_proc = subprocess.run(command, check=True, capture_output=False)

 ####################### Create Kallisto index ###############################
def kallisto_index(index_path, transcriptome_path):
    print('\033[33mRunning kallisto index! \033[0m')
    if os.path.exists(index_path):
        print("index path '%s' exists, skipping the indexing" % index_path)
        return

    command = ['kallisto',
               'index',
               '-i', index_path,
               transcriptome_path]
    kallistoindex_cmd = ' '.join(command)
    print('kallisto index command: %s' % kallistoindex_cmd)
    compl_proc = subprocess.run(command, check=True, capture_output=False)

####################### Running the Pipeline ###############################
def run_pipeline(data_folder, results_folder, genome_dir, transcriptome_file, args):
    folder_count = 1

    # Loop through each data folder
    folder_name = data_folder.split('/')[-1]
    print('\033[33mProcessing Folder: %s\033[0m' %(folder_name))

    # Get the list of first file names in paired end sequences
    first_pair_files = glob.glob('%s/*_1.fq*' %(data_folder))
    print(first_pair_files)

    # Program specific results directories
    organism = os.path.basename(genome_dir)
    data_trimmed_dir = os.path.join(results_folder, folder_name, "trimmed")
    fastqc_dir = os.path.join(results_folder, folder_name, "fastqc_results")
    results_dir = os.path.join(results_folder, folder_name, organism)
    htseq_dir = os.path.join(results_folder, "htseqcounts")

    # Run create directories function to create directory structure
    create_result_dirs(data_trimmed_dir,fastqc_dir,results_dir, htseq_dir)

    index_path = os.path.join(results_folder, "%s_kallistoindex" % organism)
    kallisto_index(index_path, transcriptome_file)

    # Loop through each file and create filenames
    file_count = 1
    for first_pair_file in first_pair_files:
        first_file_name_full = first_pair_file.split('/')[-1]

        second_pair_file = first_pair_file.replace('_1.fq', '_2.fq')
        second_file_name_full = second_pair_file.split('/')[-1]
        file_ext = first_pair_file.split('.')[-1]

        print ('\033[32m Processing File: %s of %s (%s)\033[0m' %(file_count, len(first_pair_files), first_file_name_full ))

        first_file_name = re.split('.fq|.fq.gz',first_file_name_full)[0]
        second_file_name = re.split('.fq|.fq.gz',second_file_name_full)[0]
        print('first_file_name:%s, second_file_name:%s' %(first_file_name,second_file_name))

        # Collect Sample attributes
        exp_name = folder_name
        print("exp_name: %s" %(exp_name))
        lane = first_file_name.split("_")[-1]
        print("Lane: %s" %(lane))
        sample_id = re.split('.fq|.fq.gz', first_file_name)[0]
        print("sample_id: %s" %(sample_id))

        # 01. Run TrimGalore
        # WW: Activate/deactivate for debugging
        trim_galore(first_pair_file, second_pair_file, folder_name, sample_id,
                    file_ext, data_trimmed_dir, fastqc_dir)
        file_count += 1

        # Run folder level salmon analysis
        first_pair_group, second_pair_group, pair_files = collect_trimmed_data(data_trimmed_dir,file_ext)
        run_kallisto(index_path, results_dir, pair_files)

        folder_count += 1

    return data_trimmed_dir,fastqc_dir,results_dir


DESCRIPTION = """run_kallisto.py - run Kallisto pipeline"""

if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('genomedir', help='genome directory')
    parser.add_argument('dataroot', help="parent of input directory")
    parser.add_argument('indir', help="input directory without the input dir part")
    parser.add_argument('transcriptome_file', help="path to transcriptome_file")
    parser.add_argument('outdir', help='output directory')
    args = parser.parse_args()

    print("Processing directory %s" % args.indir)
    data_folder = os.path.join(args.dataroot, args.indir)
    data_trimmed_dir, fastqc_dir, results_dir = run_pipeline(data_folder, args.outdir, args.genomedir,
                                                             args.transcriptome_file, args)



#!/usr/bin/env python3

"""
Module to encapsulate STAR indexing.

It takes the genome directory and one or more FASTA files
and passes them to STAR to generate a genome index within
the genome directory.
If the index exists, it will skip the generation, to avoid
wasting time as this is a very costly step.
"""
import argparse
import os
import subprocess
import glob


DESCRIPTION = """index_star_salmon.py - Create genome index using STAR"""


####################### Create STAR index ###############################
### This should be specific for the organism
### Use the equation file maybe another script to create references
def create_genome_index(genome_dir, genome_fasta, args):
    index_command = ['STAR', '--runMode', 'genomeGenerate',
                     '--runThreadN', str(args.runThreadN),
                     '--genomeDir', genome_dir,
                     '--genomeFastaFiles', genome_fasta,
                     '--genomeChrBinNbits', str(args.genomeChrBinNbits),
                     '--genomeSAindexNbases', str(args.genomeSAindexNbases)]
    index_cmd = ' '.join(index_command)
    print("RUNNING STAR in index MODE: '%s'" % index_cmd, flush=True)

    print ("\033[34m %s Indexing genome... \033[0m", flush=True)
    if os.path.exists('%s/SAindex' % (genome_dir)):
        print ('Genome indexes exist. Not creating!', flush=True)
    else:
        print('Creating genome indexes', flush=True)
        compl_proc = subprocess.run(index_command, check=True, capture_output=False, cwd=genome_dir)
        print('finished indexing with STAR', flush=True)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('genomedir', help='genome directory')
    parser.add_argument('--genome_fasta', help='genome FASTA file')
    parser.add_argument('--runThreadN', type=int, default=32)
    parser.add_argument('--genomeChrBinNbits', type=int, default=16)
    parser.add_argument('--genomeSAindexNbases', type=int, default=12)
    args = parser.parse_args()
    if args.genome_fasta is not None and os.path.exists(args.genome_fasta):
        genome_fasta = args.genome_fasta
    else:
        genome_fasta = glob.glob('%s/*.fasta' % (args.genomedir))[0]
    create_genome_index(args.genomedir, genome_fasta, args)



#!/usr/bin/env python3

import jinja2
import os
import argparse
import json

from globalsearch.rnaseq.find_files import rnaseq_data_folder_list

TEMPLATE = """#!/bin/bash

#SBATCH -J kallisto_{{genome}}
#SBATCH -o {{log_dir}}/"%j".out
#SBATCH -e {{log_dir}}/"%j".out
#SBATCH --array={{array_range}}

{{sbatch_options_comments}}

echo "ARRAY TASK ID: $SLURM_ARRAY_TASK_ID"
data_folders=({{data_folders}})
data_folder=${data_folders[$SLURM_ARRAY_TASK_ID]}

{{sbatch_extras}}

python3 -m globalsearch.rnaseq.run_kallisto {{genome_dir}} {{input_dir}} $data_folder {{genome_fasta}} {{output_dir}}
"""

DESCRIPTION = """make_kallisto_job.py - Create Kallisto job file for Slurm"""


def make_sbatch_options(config):
    result = ""
    for option in config['sbatch_options']['kallisto']['options']:
        result += "#SBATCH %s\n" % option
    return result

def make_sbatch_extras(config):
    result = ""
    for extra in config['sbatch_options']['kallisto']['extras']:
        result += "%s\n" % extra
    return result


if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('configfile', help="configuration file")
    args = parser.parse_args()
    with open(args.configfile) as infile:
        config = json.load(infile)

    templ = jinja2.Template(TEMPLATE)
    genome = os.path.basename(os.path.normpath(config['genome_dir']))
    config['genome'] = genome
    config['sbatch_extras'] = make_sbatch_extras(config)
    config['sbatch_options_comments'] = make_sbatch_options(config)

    data_folders = rnaseq_data_folder_list(config)
    config["data_folders"] = ' '.join(data_folders)

    # Array specification
    try:
        array_max_tasks = config['sbatch_options']['array_max_tasks']
    except:
        array_max_tasks = 0
    if array_max_tasks > 0:
        array_max_task_spec = "%%%d" % array_max_tasks
    else:
        array_max_task_spec = ""

    config["array_range"] = "0-%d%s" % (len(data_folders) - 1, array_max_task_spec)
    print(templ.render(config))



#!/usr/bin/env python3

import jinja2
import os
import argparse
import json

from globalsearch.rnaseq.find_files import rnaseq_data_folder_list

TEMPLATE = """#!/bin/bash

#SBATCH -J star_salmon_{{genome}}
#SBATCH -o {{log_dir}}/"%A"."%a".out
#SBATCH -e {{log_dir}}/"%A"."%a".out
#SBATCH --array={{array_range}}

{{sbatch_options_comments}}

echo "ARRAY TASK ID: $SLURM_ARRAY_TASK_ID"
data_folders=({{data_folders}})
data_folder=${data_folders[$SLURM_ARRAY_TASK_ID]}
star_prefix="star_{{star_options.outFilterMismatchNmax}}_{{star_options.outFilterMismatchNoverLmax}}_{{star_options.outFilterScoreMinOverLread}}_{{star_options.outFilterMatchNmin}}{{dedup_prefix}}"
salmon_prefix="salmon_{{star_options.outFilterMismatchNmax}}_{{star_options.outFilterMismatchNoverLmax}}_{{star_options.outFilterScoreMinOverLread}}_{{star_options.outFilterMatchNmin}}{{dedup_prefix}}"

{{sbatch_extras}}

python3 -m globalsearch.rnaseq.run_star_salmon {{twopass_mode}} {{fastq_patterns}} {{runThreadN}} {{out_sam_attributes}} --outFilterMismatchNmax {{star_options.outFilterMismatchNmax}} --outFilterMismatchNoverLmax {{star_options.outFilterMismatchNoverLmax}} --outFilterScoreMinOverLread {{star_options.outFilterScoreMinOverLread}} --outFilterMatchNmin {{star_options.outFilterMatchNmin}} {{dedup_option}} --starPrefix $star_prefix --salmonPrefix $salmon_prefix {{genome_gff_option}} {{genome_fasta_option}} {{genome_dir}} {{input_dir}} $data_folder {{output_dir}}
"""

DESCRIPTION = """make_star_salmon_job.py - Create STAR Salmon job file for Slurm"""

def make_sbatch_options(config):
    result = ""
    for option in config['sbatch_options']['star_salmon']['options']:
        result += "#SBATCH %s\n" % option
    return result

def make_sbatch_extras(config):
    result = ""
    for extra in config['sbatch_options']['star_salmon']['extras']:
        result += "%s\n" % extra
    return result

if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('configfile', help="configuration file")
    args = parser.parse_args()
    with open(args.configfile) as infile:
        config = json.load(infile)

    templ = jinja2.Template(TEMPLATE)
    genome = os.path.basename(os.path.normpath(config['genome_dir']))
    config['genome'] = genome
    config['sbatch_extras'] = make_sbatch_extras(config)
    config['sbatch_options_comments'] = make_sbatch_options(config)

    config['dedup_prefix'] = '_dedup' if config['deduplicate_bam_files'] else ''
    config['dedup_option'] = '--dedup' if config['deduplicate_bam_files'] else ''
    config['dedup_option'] = '--dedup' if config['deduplicate_bam_files'] else ''
    config['twopass_mode'] = '--twopassMode' if config['star_options']['twopassMode'] else ''

    # override runThreadN
    try:
        config["runThreadN"] = "--runThreadN %d" % config['star_options']['runThreadN']
    except:
        config["runThreadN"] = ''

    # override outSAMattributes
    try:
        out_sam_attributes = config['star_options']['outSAMattributes']
        config['out_sam_attributes'] = '--outSAMattributes %s' % ' '.join(out_sam_attributes) if len(out_sam_attributes) > 0 else ''
    except:
        config['out_sam_attributes'] = ''

    config['fastq_patterns'] = '--fastq_patterns "%s"' % ','.join(config['fastq_patterns']) if len(config['fastq_patterns']) > 0 else ''

    # see if optional genome_gff exists
    try:
        config['genome_gff_option'] = ''
        genome_gff = config['genome_gff']
        if os.path.exists(genome_gff):
            config['genome_gff_option'] = '--genome_gff %s' % genome_gff
        try:
            sjdb_gtf_tag_exon_parent_transcript = config['star_options']['sjdbGTFtagExonParentTranscript']
            config['genome_gff_option'] += (' --sjdbGTFtagExonParentTranscript %s' % sjdb_gtf_tag_exon_parent_transcript)
        except KeyError:
            pass  # ignore if doesn't exist
        try:
            sjdb_overhang = config['star_options']['sjdbOverhang']
            config['genome_gff_option'] += (' --sjdbOverhang %s' % str(sjdb_overhang))
        except KeyError:
            pass  # ignore if doesn't exist
        try:
            limit_sjdb_insert_nsj = config['star_options']['limitSjdbInsertNsj']
            config['genome_gff_option'] += (' --limitSjdbInsertNsj %s' % str(limit_sjdb_insert_nsj))
        except KeyError:
            pass  # ignore if doesn't exist
    except:
        pass

    # see if optional genome_fasta exists
    try:
        config['genome_fasta_option'] = ''
        genome_fasta = config['genome_fasta']
        if os.path.exists(genome_fasta):
            config['genome_fasta_option'] = '--genome_fasta %s' % genome_fasta
    except:
        pass

    data_folders = rnaseq_data_folder_list(config)
    config["data_folders"] = ' '.join(data_folders)

    # Array specification
    try:
        array_max_tasks = config['sbatch_options']['array_max_tasks']
    except:
        array_max_tasks = 0
    if array_max_tasks > 0:
        array_max_task_spec = "%%%d" % array_max_tasks
    else:
        array_max_task_spec = ""

    config["array_range"] = "0-%d%s" % (len(data_folders) - 1, array_max_task_spec)
    print(templ.render(config))



import os
import re
import codecs
from setuptools import setup

NAME = 'globalsearch'
PACKAGES = ['globalsearch.rnaseq', 'globalsearch.control']
DESCRIPTION = 'globalsearch is a collection of Python modules and command tools for the Global Search pipeline.'
LICENSE = 'LGPL V3'
URI = 'https://github.com/baliga-lab/Global_Search'
AUTHOR = 'Wei-ju Wu'
VERSION = '0.1.8'

KEYWORDS = ['global search', 'coral reef']

# See trove classifiers
# https://testpypi.python.org/pypi?%3Aaction=list_classifiers

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: Implementation :: CPython",
    "Topic :: Software Development :: Libraries :: Python Modules"
    ]
INSTALL_REQUIRES = ['jinja2', 'fs', 'xmlrunner', 'rpy2']


if __name__ == '__main__':
    setup(name=NAME, description=DESCRIPTION,
          license=LICENSE,
          url=URI,
          version=VERSION,
          author=AUTHOR,
          author_email='weiju.wu@gmail.com',
          maintainer=AUTHOR,
          maintainer_email='weiju.wu@gmail.com',
          keywords=KEYWORDS,
          packages=PACKAGES,
          zip_safe=False,
          classifiers=CLASSIFIERS,
          install_requires=INSTALL_REQUIRES,
          scripts=['bin/gs_submit', 'bin/gs_prepare'])



import os
from fs.osfs import OSFS
import fs

TRIMGALORE_SUFFIX_SINGLE = "_trimmed.fq"
TRIMGALORE_SUFFIX_PAIRED = "_val_1.fq"
PAIRED_END_FQ_PATTERN = '/*_val_%d.fq'
SINGLE_END_FQ_PATTERN = '/*_1_trimmed.fq'

"""
trim_galore --fastqc_args "--outdir /proj/omics4tb2/wwu/Global_Search/redsea-output/R1/fastqc_results/" --paired --output_dir /proj/omics4tb2/wwu/Global_Search/redsea-output/R1/trimmed/ /proj/omics4tb2/wwu/GlobalSearch.old/Pilot_Fail_Concat/rawdata/R1/R1_concat_1.fq.gz /proj/omics4tb2/wwu/GlobalSearch.old/Pilot_Fail_Concat/rawdata/R1/R1_concat_2.fq.gz"""

def trim_galore(first_pair_file, second_pair_file, folder_name, sample_id, data_trimmed_dir,
                fastqc_dir):
    paired = second_pair_file is not None  # make sure we are not single end

    # check whether the result already exists and skip if it does
    file_base = os.path.basename(first_pair_file).replace(".gz", "").replace(".fastq", "").replace(".fq", "")
    if paired:
        trimmed_file_name = file_base + TRIMGALORE_SUFFIX_PAIRED
    else:
        trimmed_file_name = file_base + TRIMGALORE_SUFFIX_SINGLE

    trimmed_path = os.path.join(data_trimmed_dir, trimmed_file_name)
    trimmed_path_gz = trimmed_path + ".gz"
    if os.path.exists(trimmed_path) or os.path.exists(trimmed_path_gz):
        print("Trimmed file '%s' found, skipping trim_galore" % trimmed_file_name, flush=True)
        return

    print ("\033[34m Running TrimGalore \033[0m")
    # create sample specific trimmed and fastqc directories
    if not os.path.exists(data_trimmed_dir):
        os.makedirs(data_trimmed_dir)
    if not os.path.exists(fastqc_dir):
        os.makedirs(fastqc_dir)

    # run Command
    command = ['trim_galore', '--fastqc_args', '"--outdir %s/"' % fastqc_dir]
    if paired:
        command.append('--paired')
    command += ['--output_dir', data_trimmed_dir, first_pair_file]
    if second_pair_file is not None:
        command.append(second_pair_file)
    cmd = ' '.join(command)
    print( '++++++ Trimgalore Command:', cmd)
    os.system(cmd)  # run with os.system(), since you have that funny outdir parameter


####################### Collect trimmed data files ###############################


def collect_trimmed_data(data_trimmed_dir, is_gzip, is_paired_end, rootfs=OSFS("/")):
    """
    Collect trimmed data files from the specified trimmed directory
    """
    filesys = rootfs.opendir(data_trimmed_dir)
    # define result files
    pattern = PAIRED_END_FQ_PATTERN if is_paired_end else SINGLE_END_FQ_PATTERN
    if is_gzip:
        pattern += ".gz"

    if is_paired_end:
        first_pair_trimmed, second_pair_trimmed = [filesys.glob(pattern % i) for i in [1, 2]]
    else:
        first_pair_trimmed = filesys.glob(pattern)
        second_pair_trimmed = None

    first_pair_trimmed = [fs.path.combine(data_trimmed_dir, match.path)
                          for match in first_pair_trimmed]
    if second_pair_trimmed is not None:
        second_pair_trimmed = [fs.path.combine(data_trimmed_dir, match.path)
                               for match in second_pair_trimmed]
    else:
        second_pair_trimmed = []

    first_pair_group = ' '.join(first_pair_trimmed)
    second_pair_group = ' '.join(second_pair_trimmed)

    return first_pair_group, second_pair_group


def create_result_dirs(data_trimmed_dir, fastqc_dir, results_dir, htseq_dir):
    dirs = [data_trimmed_dir, fastqc_dir, results_dir, htseq_dir]
    for dir in dirs:
        # create results folder
        if not os.path.exists('%s' %(dir)):
            os.makedirs('%s' %(dir))
        else:
            print('\033[31m %s directory exists. Not creating. \033[0m' %(dir))



#!/usr/bin/env python3

#############################################################
##### RNASeq Analysis Pipeline - SplAdder               #####
##### Last update: 2023/03/01 by Yaqiao Li              #####
##### Institute for Systems Biology                     #####
#############################################################
import glob, sys, os, string, datetime, re, errno, time
import argparse
from multiprocessing import Pool, cpu_count, Manager
from subprocess import *
#from pathos.multiprocessing import ProcessingPoll as Pool
from functools import partial
#from find_files import find_fastq_files

DESCRIPTION = """run_spladder.py - run spladder"""

####################### Create results directories ###############################
def create_dirs(spladder_work_dir, spladder_out_dir, parsed_event_dir):
    dirs = [spladder_work_dir, spladder_out_dir, parsed_event_dir]
    for dir in dirs:
        if not os.path.exists('%s' %(dir)):
            os.makedirs('%s' %(dir))
        else:
            print('\033[31m %s directory exists. Not creating. \033[0m' %(dir))



####################### SplAdder Step1 Single graphs #############################
def SplAdder_step1_single_graphs(input_bam, genome_annotation, spladder_out_dir):
    command = ['spladder', 'build',
                '-o', spladder_out_dir,
                '-a', genome_annotation,
                '-b', input_bam,
                '--merge-strat', 'single',
                '--no-extract-ase',
                '--parallel', '2',
                '-v']
    cmd = ' '.join(command)
    print('SplAdder_Step1_single_graphs_run_command:\n%s' % cmd)
    outOut = open('stdout.out','a')
    errOut = open('ErrorReport.txt','a')
    errOut.write('\nSplAdder_Step1_single_graphs_run_command:\n%s\n' % cmd)
    spladderStep1Proc = Popen(cmd, shell=True, stdout=outOut, stderr=errOut)
    stdoutput = spladderStep1Proc.communicate()[0]
    outOut.close()
    errOut.close()
    
####################### SplAdder Step2 Merged graphs #############################
def SplAdder_step2_merged_graphs(genome_annotation, spladder_out_dir, input_bam_list):
    command = ['spladder', 'build',
                '-o', spladder_out_dir,
                '-a', genome_annotation,
                '-b', input_bam_list,
                '--merge-strat', 'merge_graphs',
                '--no-extract-ase',
                '--parallel', '40',
                '-v']
    cmd = ' '.join(command)
    print('SplAdder_Step2_merging_splice_graphs_run_command:\n%s' % cmd)
    outOut = open('stdout.out','a')
    errOut = open('ErrorReport.txt','a')
    errOut.write('\nSplAdder_Step2_merging_splice_graphs_run_command:\n%s\n' % cmd)
    spladderStep2Proc = Popen(cmd, shell=True, stdout=outOut, stderr=errOut)
    stdoutput = spladderStep2Proc.communicate()[0]
    outOut.close()
    errOut.close()
    #os.system(cmd)
    #compl_proc = subprocess.run(command, check=True, capture_output=False)

####################### SplAdder Step3 Quantification #############################
def SplAdder_step3_quantification(input_bam, genome_annotation, spladder_out_dir):
    command = ['spladder', 'build',
                '-o', spladder_out_dir,
                '-a', genome_annotation,
                '-b', input_bam,
                '--merge-strat', 'merge_graphs',
                '--no-extract-ase',
                '--quantify-graph',
                '--qmode', 'single',
                '--parallel', '2',
                '-v']
    cmd = ' '.join(command)
    print('SplAdder_Step3_splice_graphs_run_command:\n%s' % cmd)
    outOut = open('stdout.out','a')
    errOut = open('ErrorReport.txt','a')
    errOut.write('\nSplAdder_Step3_splice_graphs_run_command:\n%s\n' % cmd)
    spladderStep3Proc = Popen(cmd, shell=True, stdout=outOut, stderr=errOut)
    stdoutput = spladderStep3Proc.communicate()[0]
    outOut.close()
    errOut.close()

####################### SplAdder Step4 Aggregate Quantification####################
def SplAdder_step4_aggregate_quantification(genome_annotation, spladder_out_dir, input_bam_list):
    command = ['spladder', 'build',
                '-o', spladder_out_dir,
                '-a', genome_annotation,
                '-b', input_bam_list,
                '--merge-strat', 'merge_graphs',
                '--no-extract-ase',
                '--quantify-graph',
                '--qmode', 'collect',
                '--parallel', '40',
                '-v']
    cmd = ' '.join(command)
    print('SplAdder_Step4_aggregate_quantification_run_command:\n%s' % cmd)
    outOut = open('stdout.out','a')
    errOut = open('ErrorReport.txt','a')
    errOut.write('\nSplAdder_Step4_aggregate_quantification_run_command:\n%s\n' % cmd)
    spladderStep4Proc = Popen(cmd, shell=True, stdout=outOut, stderr=errOut)
    stdoutput = spladderStep4Proc.communicate()[0]
    outOut.close()
    errOut.close()
    #os.system(cmd)

####################### SplAdder Step5 Call Events ###############################
def SplAdder_step5_call_events(genome_annotation, spladder_out_dir, input_bam_list):
    command = ['spladder', 'build',
                '-o', spladder_out_dir,
                '-a', genome_annotation,
                '-b', input_bam_list,
                '--parallel', '40',
                '-v']
    cmd = ' '.join(command)
    print('SplAdder_Step5_call_events_run_command:\n%s' % cmd)
    outOut = open('stdout.out','a')
    errOut = open('ErrorReport.txt','a')
    errOut.write('\nSplAdder_Step5_call_events_run_command:\n%s\n' % cmd)
    spladderStep5Proc = Popen(cmd, shell=True, stdout=outOut, stderr=errOut)
    stdoutput = spladderStep5Proc.communicate()[0]
    outOut.close()
    errOut.close()
    #os.system(cmd)

####################### SplAdder Step6 Contrast Tests ############################
def SplAdder_step6_contrast_test(contrs, spladder_out_dir, contrast_dir):
    [var1, var2, const] = contrs.split()
    conditionA = contrast_dir + '/host_' + var1 + '_' + const + '.txt'
    conditionB = contrast_dir + '/host_' + var2 + '_' + const + '.txt'
    labelA = var1 + const
    labelB = var2 + const

    command = ['spladder', 'test',
                '-o', spladder_out_dir,
                '--out-tag', 'Rem_Prob',
                '--conditionA', conditionA,
                '--conditionB', conditionB,
                '--labelA', labelA,
                '--labelB', labelB,
                '--diagnose-plots',
                '--parallel', '5',
                '-v']
    cmd = ' '.join(command)
    print('SplAdder_Step6_contrast_test_run_command:\n%s' % cmd)
    outOut = open('stdout.out','a')
    errOut = open('ErrorReport.txt','a')
    errOut.write('\nSplAdder_Step6_contrast_test_run_command:\n%s\n' % cmd)
    spladderStep6Proc = Popen(cmd, shell=True, stdout=outOut, stderr=errOut)
    stdoutput = spladderStep6Proc.communicate()[0]
    outOut.close()
    errOut.close()

####################### SplAdder Step7 Parsing Statistic #########################
def SplAdder_step7_parsing_statistic(contrs, parseRscript):
    [var1, var2, const] = contrs.split()

    command = ['Rscript',
                parseRscript,
                var1,
                var2,
                const]
    cmd = ' '.join(command)
    print('SplAdder_Step7_parsing_statistic_run_command:\n%s' % cmd)
    outOut = open('stdout.out','a')
    errOut = open('ErrorReport.txt','a')
    errOut.write('\nSplAdder_Step7_parsing_statistic_run_command:\n%s\n' % cmd)
    spladderStep7Proc = Popen(cmd, shell=True, stdout=outOut, stderr=errOut)
    stdoutput = spladderStep7Proc.communicate()[0]
    outOut.close()
    errOut.close()


####################### Check Error Report  #####################################
def Check_Error_Report():
    fileopen = open('ErrorReport.txt', 'r')
    fileread = fileopen.read().replace('FileExistsError', '')
    fileopen.close()
    errOut = open('ErrorReport.txt','a')
    errorKeyWord = ['Execution halted', 'Error', 'error']
    cleanUp = True
    cf = open('cleanUp.sh','w')
    cf.write('#Remove temporary files\n\n')
    for e in errorKeyWord:
        if e in fileread:
            errOut.write('\nFind [ ' + e + ' ] error in report, please check it. For clean up manually please run cleanUp.sh\n')
            cleanUp = False
            for root, dirs, files in os.walk(spladder_out_dir):
                for f in files:
                    if f.endswith('pickle'):
                        cf.write('rm ' + os.path.join(root, f) + '\n')
    errOut.close()
    cf.close()
    return cleanUp

####################### Clean Up temporary files  ###############################
def Clean_Up(spladder_out_dir):
    for root, dirs, files in os.walk(spladder_out_dir):
        for f in files:
            if f.endswith('pickle'):
                os.remove(os.path.join(root, f))

####################### Run SplAdder ############################################
def run_spladder(spladder_work_dir, spladder_out_dir, parsed_event_dir, input_bam_list, genome_annotation, all_contrasts, contrast_dir, sampleType, args):
    # Run create directories function to create directory structure
    create_dirs(spladder_work_dir, spladder_out_dir, parsed_event_dir)
    
    #---run step 1
    bamlist = open(input_bam_list, 'r')
    bams = []
    for line in bamlist:
        single_bam = line.strip()
        bams.append(single_bam)
    print(bams)
    bamlist.close()
   
    partial_work_step1 = partial(SplAdder_step1_single_graphs, genome_annotation=genome_annotation, spladder_out_dir=spladder_out_dir)
    cpus = 16
    pool = Pool(processes=cpus)
    pool.map(partial_work_step1, bams)
    pool.close()
    pool.join()

    #---run step 2
    SplAdder_step2_merged_graphs(genome_annotation, spladder_out_dir, input_bam_list)

    #---run step 3
    partial_work_step3 = partial(SplAdder_step3_quantification, genome_annotation=genome_annotation, spladder_out_dir=spladder_out_dir)
    cpus = 16
    pool = Pool(processes=cpus)
    pool.map(partial_work_step3, bams)
    pool.close()
    pool.join()

    #---run step 4
    SplAdder_step4_aggregate_quantification(genome_annotation, spladder_out_dir, input_bam_list)

    #---run step 5
    SplAdder_step5_call_events(genome_annotation, spladder_out_dir, input_bam_list)

    #---run step 6
    contrast_list = open(all_contrasts, 'r')
    contrs = []
    for line in contrast_list:
        contrs.append(line.strip())
    contrast_list.close()

    partial_work_step6 = partial(SplAdder_step6_contrast_test, spladder_out_dir=spladder_out_dir, contrast_dir=contrast_dir)
    cpus = 16
    pool = Pool(processes=cpus)
    pool.map(partial_work_step6, contrs)
    pool.close()
    pool.join()



    #---run step 7
    parseRscript = spladder_out_dir + '/run_parse_spladder_output.R'

    strR1 = r'''args<-commandArgs(TRUE)
var1<-args[1]
var2 <-args[2]
const <-args[3]

# read in the event files for each contrast
setwd(paste0("'''

    strR2 = spladder_out_dir

    strR3 = r'''/testing_",var1,const,"_vs_",var2,const,"_Rem_Prob"))

ldf <- list()
listcsv <- dir(pattern = "*gene_unique.tsv")
for (k in 1:length(listcsv)){
    ldf[[k]] <- read.table(file = listcsv[k], header = T)
}

# combine into one DF
one_ldf <- do.call(rbind, ldf)

#add a column with the constant
one_ldf$const <- paste0(const)

#add a column with the contrast variables
one_ldf$vrs <- paste0(var1,"vs",var2)

# add a column for species name
one_ldf$spec <- c("'''

    strR3_1 = r'''")

#write out the results as one table
write.table(one_ldf, paste0("'''

    if(sampleType == 'host'):
        strR4 = parsed_event_dir + '/host_all_events'
    elif(sampleType == 'sym'):
        strR4 = parsed_event_dir + '/sym_all_events'
    else:
        strR4 = parsed_event_dir + '/other_all_events'
    strR5 = r'",var1,const,"_vs_",var2,const,".tsv"), quote = F, row.names = F)'
    strR = strR1 + strR2 + strR3 + sampleType + strR3_1 + strR4 + strR5 + '\n'
    parseR = open(parseRscript, 'w')
    parseR.write(strR)
    parseR.close()

    partial_work_step7 = partial(SplAdder_step7_parsing_statistic, parseRscript=parseRscript)
    cpus = 16
    pool = Pool(processes=cpus)
    pool.map(partial_work_step7, contrs)
    pool.close()
    pool.join()    

    #---run spladder program finished
    




####################### Main ####################################################
if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('-w', '--WorkDir', help='work directory')
    parser.add_argument('-i', '--inputBamFileList', help='input .bam file name list')
    parser.add_argument('-a', '--genomeAnnotation', help='genome GFF/GTF file')
    parser.add_argument('-c', '--allContrasts', help='sample contrasts information: all_contrasts.txt')
    parser.add_argument('-d', '--contrastDir', help='sample contrasts information: contrast_file')
    parser.add_argument('-s', '--sampleType', help='sample is host or symbiont or Other')
    
    args = parser.parse_args()
    
    work_dir = args.WorkDir
    input_bam_list = args.inputBamFileList
    genome_annotation = args.genomeAnnotation
    all_contrasts = args.allContrasts
    contrast_dir = args.contrastDir
    sampleType = args.sampleType
    
    spladder_work_dir = work_dir + '/' + sampleType + '_spladder_jobs'
    spladder_out_dir = work_dir + '/' + sampleType + '_spladder_jobs' + '/array_spladder_out'
    if sampleType == 'host' or sampleType == 'sym':
        parsed_event_dir = work_dir + '/host_sym_parsed_event_files'
    else:
        parsed_event_dir = work_dir + '/' + sampleType + '_parsed_event_files'

    errOut = open('ErrorReport.txt','w')
    starttime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
    errOut.write('Start: ' + starttime + '\n\n')
    errOut.close()
    run_spladder(spladder_work_dir, spladder_out_dir, parsed_event_dir, input_bam_list, genome_annotation, all_contrasts, contrast_dir, sampleType, args)
    errOut = open('ErrorReport.txt','a')
    endtime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
    errOut.write('\nEnd: ' + endtime + '\n\n')
    errOut.close()
    
    # check errors
    cleanUp = Check_Error_Report()
    
    # clean up
    errOut = open('ErrorReport.txt','a')
    if cleanUp:
        Clean_Up(spladder_out_dir)
        cleantime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        errOut.write('\nCleanedUp: ' + cleantime + '\n\n')
    else:
        cleantime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        errOut.write('\nCan not cleanedUp: ' + cleantime + '\n\n')     
    errOut.close()







#!/usr/bin/env python3 

"""
Script to check parameters and prepare for submission
"""
import argparse
import json
import os, sys, glob, subprocess
from rpy2.robjects.packages import importr, PackageNotInstalledError
import rpy2


DESCRIPTION = """gs_prepare.py - prepare data for workflow submission"""

OUTSAM_ATTRS_STD = {
    "NH", "HI", "AS", "nM", "NM", "MD", "jM", "jI", "XS", "MC", "ch"
}
OUTSAM_ATTRS_EXT = { "vA", "vG", "vW", "CR", "CY", "UR", "UY" }
OUTSAM_ATTRS_EXT2 = { "rB", "vR" }
OUTSAM_ATTRS_SPECIAL = { "None", "Standard", "All" }
OUTSAM_ATTRS_MULTI = OUTSAM_ATTRS_STD | OUTSAM_ATTRS_EXT | OUTSAM_ATTRS_EXT2
OUTSAM_ATTRS_SINGLE = OUTSAM_ATTRS_STD | OUTSAM_ATTRS_EXT | OUTSAM_ATTRS_EXT2 | OUTSAM_ATTRS_SPECIAL

def silent_rpy2_print(o):
    pass

rpy2.rinterface_lib.callbacks.consolewrite_print = silent_rpy2_print
rpy2.rinterface_lib.callbacks.consolewrite_warnerror = silent_rpy2_print


def check_star_options(star_options):
    try:
        outsam_attrs = star_options['outSAMattributes']
        if not isinstance(outsam_attrs, list):
            raise TypeError("STAR options: outSAMattributes expected type: 'list' but was '%s'" % type(outsam_attrs))
        if len(outsam_attrs) == 1:
            if not outsam_attrs[0] in OUTSAM_ATTRS_SINGLE:
                raise ValueError("STAR options (single): outSAMattributes: '%s'" % [outsam_attrs[0]])
        else:
            for attr in outsam_attrs:
                if not attr in OUTSAM_ATTRS_MULTI:
                    raise ValueError("STAR options (multiple): outSAMattributes: '%s'" % attr)
    except KeyError:
        # not specified -> doesn't matter
        pass


def check_params(config, rna_algo):
    """ensures integrity of the parameters"""
    print("checking integrity of parameters...", end="", flush=True)
    if not os.path.exists(config["input_dir"]):
        sys.exit("Input directory '%s' does not exist" % config["input_dir"])

    try:
        if len(config['genome_fasta'].strip()) > 0 and not os.path.exists(config["genome_fasta"]):
            sys.exit("ERROR: Specified Genome FASTA '%s' does not exist" % config["genome_fasta"])
    except:
        print("WARNING: genome_gff not specified (htseq will be skipped)", flush=True)

    if not os.path.exists(config["genome_dir"]):
        sys.exit("Genome directory '%s' does not exist" % config["genome_dir"])
        genome_fasta = glob.glob('%s/*.fasta' % (args.genomedir))
        if len(genome_fasta) == 0:
            sys.exit("Genome directory '%s' does not contain any FASTA files" % config["genome_dir"])

    if len(config["includes"]) > 0:
        # check the existence of the included directories
        for incl in config["includes"]:
            inp_dir = os.path.join(config["input_dir"], incl)
            if not os.path.exists(inp_dir):
                sys.exit("Input directory '%s' does not exist" % inp_dir)
    try:
        check_star_options(config['star_options'])
    except ValueError as e:
        sys.exit(str(e))
    except KeyError:
        pass

    print("done.", flush=True)


def create_dirs(config):
    if not os.path.exists(config["output_dir"]):
        print("creating output directory '%s'" % config["output_dir"], flush=True)
        os.makedirs(config["output_dir"])
    if not os.path.exists(config["log_dir"]):
        print("creating log directory '%s'" % config["log_dir"], flush=True)
        os.makedirs(config["log_dir"])


def __check_command(command, num_info_components=1, version_index=-1,
                    check_version=None, version_switch='--version',
                    multiline=False, info_line=0, fail_if_not_exists=True):
    """Generic command checker, can check for different version info formats and
    restrict version numbers
    """
    try:
        print("checking for %s... " % command, end="", flush=True)
        compl_proc = subprocess.run([command, version_switch], check=True, capture_output=True)
        if multiline:
            info_string = compl_proc.stdout.decode('utf-8').split('\n')[info_line].strip()
        else:
            info_string = compl_proc.stdout.decode('utf-8').strip()
        comps = info_string.split()
        version = comps[version_index]

        print("(found version '%s') ..." % version, end="", flush=True)
        if check_version is not None and check_version != version:
            sys.exit("Unsupported version %s. Currently, only %s %s is supported" % (command, check_version))
        print("done.", flush=True)
    except FileNotFoundError:
        if fail_if_not_exists:
            sys.exit("Can not find %s (not installed or not in PATH)" % command)
        else:
            print("WARN: %s does not exist, but is optional" % command, flush=True)


def check_salmon():
    __check_command("salmon", num_info_components=2, check_version="0.13.1")


def check_star():
    __check_command("STAR")


def check_htseq():
    __check_command("htseq-count")


def check_samtools():
    __check_command("samtools", multiline=True, num_info_components=2)


def check_kallisto():
    __check_command("kallisto", version_switch="version", num_info_components=3,
                    fail_if_not_exists=False)

def check_trim_galore():
    __check_command("trim_galore", num_info_components=2, multiline=True, info_line=3)


def check_rlibrary_installed(libname):
    try:
        lib = importr(libname)
        return True
    except PackageNotInstalledError:
        return False


def check_rlibraries_installed():
    print("checking for installed R libraries...", end="", flush=True)
    not_installed = []
    for rlib in ['GlobalSearch']:
        if not check_rlibrary_installed(rlib):
            not_installed.append(rlib)
    print("done", flush=True)

    return not_installed

if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('configfile', help="configuration file")
    args = parser.parse_args()

    with open(args.configfile) as infile:
        config = json.load(infile)
    rna_algo = config['rnaseq_algorithm']

    if rna_algo == 'star_salmon':
        check_star()
        check_salmon()
    if rna_algo == 'kallisto':
        check_kallisto()

    check_htseq()
    check_samtools()
    check_trim_galore()
    not_installed = check_rlibraries_installed()
    if len(not_installed) > 0:
        for libname in not_installed:
            print("ERROR: Please install the R library '%s'" % libname)
        sys.exit()

    check_params(config, rna_algo)
    create_dirs(config)
    print(rna_algo)



#!/usr/bin/env python3

"""
Generate a SLURM job file to call the STAR indexer
It accepts a pipeline configuration file and
fills out the template with the appropriate parameters
"""

import jinja2
import os
import argparse
import json


TEMPLATE = """#!/bin/bash

#SBATCH -J star_salmon_{{genome}}
#SBATCH -o {{log_dir}}/"%j".out
#SBATCH -e {{log_dir}}/"%j".out

{{sbatch_options}}

echo "TASK ID: $SLURM_JOB_ID"

{{sbatch_extras}}

python3 -m globalsearch.rnaseq.index_star {{star_index_cmd_options}} {{genome_fasta_option}} {{genome_dir}}
"""

DESCRIPTION = """make_star_salmon_job.py - Create STAR Salmon job file for Slurm"""

def make_sbatch_options(config):
    result = ""
    for option in config['sbatch_options']['star_salmon']['options']:
        result += "#SBATCH %s\n" % option
    return result

def make_sbatch_extras(config):
    result = ""
    for extra in config['sbatch_options']['star_salmon']['extras']:
        result += "%s\n" % extra
    return result

if __name__ == '__main__':
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=DESCRIPTION)
    parser.add_argument('configfile', help="configuration file")
    args = parser.parse_args()
    with open(args.configfile) as infile:
        config = json.load(infile)

    templ = jinja2.Template(TEMPLATE)
    genome = os.path.basename(os.path.normpath(config['genome_dir']))
    config['genome'] = genome
    config['sbatch_extras'] = make_sbatch_extras(config)
    config['sbatch_options'] = make_sbatch_options(config)

    # see if optional genome_fasta exists
    try:
        config['genome_fasta_option'] = ''
        genome_fasta = config['genome_fasta']
        if os.path.exists(genome_fasta):
            config['genome_fasta_option'] = '--genome_fasta %s' % genome_fasta
    except:
        pass

    # see if optional star_index_options exists
    try:
        config['star_index_cmd_options'] = ''
        star_index_options = config['star_index_options']
        options = []
        try:
            options.append("--runThreadN %d" % star_index_options['runThreadN'])
        except KeyError:
            pass
        try:
            options.append("--genomeChrBinNbits %d" % star_index_options['genomeChrBinNbits'])
        except KeyError:
            pass
        try:
            options.append("--genomeSAindexNbases %d" % star_index_options['genomeSAindexNbases'])
        except KeyError:
            pass
        config['star_index_cmd_options'] = ' '.join(options)
    except KeyError:
        pass

    print(templ.render(config))
